'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _babelTypesPatch = require('./util/babelTypesPatch');

var _babelTypesPatch2 = _interopRequireDefault(_babelTypesPatch);

var _lscNodeTypes = require('./lscNodeTypes');

var _helpers = require('./state/helpers');

var _config = require('./config');

var _findBabelOptionsForPlugin = require('./util/findBabelOptionsForPlugin');

var _findBabelOptionsForPlugin2 = _interopRequireDefault(_findBabelOptionsForPlugin);

var _compilerState = require('./state/compilerState');

var _fixAst = require('./visitors/fixAst');

var _fixAst2 = _interopRequireDefault(_fixAst);

var _placeholders = require('./visitors/placeholders');

var _placeholders2 = _interopRequireDefault(_placeholders);

var _optionalChains = require('./visitors/optionalChains');

var _optionalChains2 = _interopRequireDefault(_optionalChains);

var _main = require('./visitors/main');

var _main2 = _interopRequireDefault(_main);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// TODO: factor out to config.lsc
function computeParserOpts(file, pluginOpts) {
  // By this time, we have access to the code of the file.
  // We can parse shebang, directives, etc and manipulate parser options
  // based on those data.
  file.parseShebang();
  var opts = file.opts,
      parserOpts = file.parserOpts;

  var fileTypeInfo = (0, _compilerState.getFileTypeInfo)(file);
  var cdOpts = (0, _config.parseConfigurationDirectives)(file.code);

  // Don't read as LSC
  if (!fileTypeInfo.isLightScript && !cdOpts.isLightScript) return;

  file.isLightScript = true;
  Object.assign(pluginOpts, cdOpts);

  // Get LSC parser opts
  opts.parserOpts = opts.parserOpts || {};
  var nextParserOpts = (0, _config.getParserOpts)(pluginOpts, opts.parserOpts);
  parserOpts.parse = nextParserOpts.parse;
  return parserOpts.plugins = nextParserOpts.plugins;
} // XXX: should be part of state init

exports.default = function LightScript(babel) {
  var t = babel.types;

  (0, _babelTypesPatch2.default)(t);
  (0, _lscNodeTypes.registerLightscriptNodeTypes)(t);

  var plugin = {
    manipulateOptions(opts, parserOpts, file) {
      // manipulateOptions is called very early in the Babel processing chain.
      // It doesn't get an opportunity to see the configuration options for the
      // plugin being run (without a workaround) nor does it get an
      // opportunity to see the code that will be compiled.
      //
      // This rules out things like parsing configuration directives from the
      // top of a JS file to manipulate parsing and compiling options.
      //
      // To remedy this, we monkey patch so that we can do initialization work
      // after the code is read, but before the parser runs.
      //
      // Relevant Babel code:
      // https://github.com/babel/babel/blob/4e50b2d9d9c376cee7a2cbf56553fe5b982ea53c/packages/babel-core/src/transformation/file/index.js#L63
      // https://github.com/babel/babel/blob/4e50b2d9d9c376cee7a2cbf56553fe5b982ea53c/packages/babel-core/src/transformation/file/index.js#L483
      var origAddCode = file.addCode;

      return file.addCode = function addCode(code) {
        origAddCode.call(this, code);
        var pluginOpts = (0, _findBabelOptionsForPlugin2.default)(this.opts, plugin);

        // compiler = createCompilerState()
        // this.lscCompiler = compiler
        // compiler~preParseInitialization(this, code, this.opts, pluginOpts)
        computeParserOpts(this, pluginOpts);
        return;
      };
    }, visitor: {
      // The compiler's main loop is a Program visitor.
      Program(path, state) {
        ////////// Initialize compilation run
        // Obtain configuration
        var opts = state.opts;
        var compiler = (0, _compilerState.createCompilerState)({
          programPath: path,
          babel,
          opts,
          file: state.file
        });

        // Early-out if not a LightScript program
        if (!compiler.isLightScript) return;

        (0, _compilerState.initializeCompilerState)();

        // TODO: helpers -> compilerState
        (0, _helpers.resetHelpers)(path);

        ////////// AST visitation and transformation
        // First pass: Perform basic ast fixups (block bodies, etc)
        (0, _fixAst2.default)(compiler, path);
        // Second pass: transform placeholders
        if (compiler.opts.placeholderArgs) {
          (0, _placeholders2.default)(compiler, path);
        } // Third pass: transform optional chains
        (0, _optionalChains2.default)(compiler, path);
        // Main ast visitor
        (0, _main2.default)(compiler, path);

        ////////// Post-processing
        // Fix up loose ends like adding `import` statements
        (0, _compilerState.postprocess)(compiler);
      } }
  };
  return plugin;
};

module.exports = exports['default'];